#include <stdio.h>
#include <stdlib.h>

#pragma pack(push)  /* push current alignment to stack */
#pragma pack(1)     /* set alignment to 1 byte boundary */

typedef struct {
   unsigned short int type;                 /* Magic identifier            */
   unsigned int size;                       /* File size in bytes          */
   unsigned short int reserved1, reserved2;
   unsigned int offset;                     /* Offset to image data, bytes */
} HEADER;

typedef struct {
   unsigned int size;               /* Header size in bytes      */
   int width,height;                /* Width and height of image */
   unsigned short int planes;       /* Number of colour planes   */
   unsigned short int bits;         /* Bits per pixel            */
   unsigned int compression;        /* Compression type          */
   unsigned int imagesize;          /* Image size in bytes       */
   int xresolution,yresolution;     /* Pixels per meter          */
   unsigned int ncolours;           /* Number of colours         */
   unsigned int importantcolours;   /* Important colours         */
} INFOHEADER;

#pragma pack(pop)   /* restore original alignment from stack */
//#  ifndef WIN32
//typedef struct                       /**** BMP file header structure ****/
//    {
//    unsigned short bfType;           /* Magic number for file */
//    unsigned int   bfSize;           /* Size of file */
//    unsigned short bfReserved1;      /* Reserved */
//    unsigned short bfReserved2;      /* ... */
//    unsigned int   bfOffBits;        /* Offset to bitmap data */
//    } BITMAPFILEHEADER;
//
//#  define BF_TYPE 0x4D42             /* "MB" */
//
//typedef struct                       /**** BMP file info structure ****/
//    {
//    unsigned int   biSize;           /* Size of info header */
//    int            biWidth;          /* Width of image */
//    int            biHeight;         /* Height of image */
//    unsigned short biPlanes;         /* Number of color planes */
//    unsigned short biBitCount;       /* Number of bits per pixel */
//    unsigned int   biCompression;    /* Type of compression to use */
//    unsigned int   biSizeImage;      /* Size of image data */
//    int            biXPelsPerMeter;  /* X pixels per meter */
//    int            biYPelsPerMeter;  /* Y pixels per meter */
//    unsigned int   biClrUsed;        /* Number of colors used */
//    unsigned int   biClrImportant;   /* Number of important colors */
//    } BITMAPINFOHEADER;


int output(const char*pszFileName, unsigned short* dst)
{

	FILE*stream = fopen(pszFileName, "r+b");
	if (stream != NULL)
	{
		HEADER head = {};
		if (fread(&head, sizeof(HEADER),1, stream) == 1)
		{
			if (head.type == 'MB')
			{
				INFOHEADER info = {};
				printf("infosize: %d, offset: %d", sizeof(INFOHEADER), head.offset - sizeof(INFOHEADER));
				if (fread(&info, sizeof(INFOHEADER),1, stream) == 1)
				{
					printf("compression:%d, bits:%d, ncolors:%d\n", info.compression, info.bits , info.ncolours);
					if (info.compression == 0 && info.bits == 24 && info.ncolours == 0)
					{
						fseek(stream, head.offset, SEEK_SET);
						for (unsigned int y = 0; y < info.height; y ++)
						{
							for (unsigned int x = 0; x < info.width; x ++)
							{
								unsigned short r = 0, g = 0, b = 0;
								r = fgetc(stream) >> 3;
								g = fgetc(stream) >> 2;
								b = fgetc(stream) >> 3;

								dst[0] = r << 11;
								dst[0] |= g << 5;
								dst[0] |= b ;
								dst ++;
							}
							dst += 1920-info.width;
						}
					}
				}
			}

		}
		fclose(stream);
	}
}
